@@ -71,6 +71,7 @@ This is implemented as new class TableHeader with the properties:
 * external: A bool indicating whether the header is outside the table cells.
 
 """
+
 import inspect
 import itertools
 import string
@@ -361,7 +362,7 @@ class WordExtractor:
         keep_blank_chars: bool = False,
         use_text_flow=False,
         horizontal_ltr=True,  # Should words be read left-to-right?
-        vertical_ttb=True,  # Should vertical words be read top-to-bottom?
+        vertical_ttb=False,  # Should vertical words be read top-to-bottom?
         extra_attrs=None,
         split_at_punctuation=False,
         expand_ligatures=True,
@@ -387,9 +388,20 @@ class WordExtractor:
         x0, top, x1, bottom = objects_to_bbox(ordered_chars)
         doctop_adj = ordered_chars[0]["doctop"] - ordered_chars[0]["top"]
         upright = ordered_chars[0]["upright"]
-
         direction = 1 if (self.horizontal_ltr if upright else self.vertical_ttb) else -1
 
+        matrix = ordered_chars[0]["matrix"]
+
+        rotation = 0
+        if not upright and matrix[1] < 0:
+            ordered_chars = reversed(ordered_chars)
+            rotation = 270
+
+        if matrix[0] < 0 and matrix[3] < 0:
+            rotation = 180
+        elif matrix[1] > 0:
+            rotation = 90
+
         word = {
             "text": "".join(
                 self.expansions.get(c["text"], c["text"]) for c in ordered_chars
@@ -401,6 +413,7 @@ class WordExtractor:
             "bottom": bottom,
             "upright": upright,
             "direction": direction,
+            "rotation": rotation,
         }
 
         for key in self.extra_attrs:
@@ -552,7 +565,8 @@ class WordExtractor:
         return WordMap(list(self.iter_extract_tuples(chars)))
 
     def extract_words(self, chars: list) -> list:
-        return list(word for word, word_chars in self.iter_extract_tuples(chars))
+        words = list(word for word, word_chars in self.iter_extract_tuples(chars))
+        return words
 
 
 def extract_words(chars: list, **kwargs) -> list:
@@ -590,8 +604,21 @@ def extract_text(chars: list, **kwargs) -> str:
             **{k: kwargs[k] for k in WORD_EXTRACTOR_KWARGS if k in kwargs}
         )
         words = extractor.extract_words(chars)
-        lines = cluster_objects(words, itemgetter("doctop"), y_tolerance)
-        return "\n".join(" ".join(word["text"] for word in line) for line in lines)
+        rotation = words[0]["rotation"]  # rotation cannot change within a cell
+
+        if rotation == 90:
+            words.sort(key=lambda w: (w["x1"], -w["top"]))
+            lines = " ".join([w["text"] for w in words])
+        elif rotation == 270:
+            words.sort(key=lambda w: (-w["x1"], w["top"]))
+            lines = " ".join([w["text"] for w in words])
+        else:
+            lines = cluster_objects(words, itemgetter("doctop"), y_tolerance)
+            lines = "\n".join(" ".join(word["text"] for word in line) for line in lines)
+            if rotation == 180:  # needs extra treatment
+                lines = "".join([(c if c != "\n" else " ") for c in reversed(lines)])
+
+        return lines
 
 
 def collate_line(
@@ -1406,7 +1433,7 @@ class Table(object):
 
             l_r = set()  # (x0, x1) pairs for all table cells
             for cell in table.cells:
-                if cell == None:  # skip non-existing cells
+                if cell is None:  # skip non-existing cells
                     continue
                 cellbb = Rect(cell)
 
@@ -1425,9 +1452,6 @@ class Table(object):
                     cells.append((x0, y0, x1, y1))
             return cells, bbox
 
-        # we depend on small glyph heights!
-        old_small = TOOLS.set_small_glyph_heights()
-        TOOLS.set_small_glyph_heights(True)
         try:
             row = self.rows[0]
             cells = row.cells
@@ -1572,7 +1596,6 @@ class Table(object):
             page.get_textbox(c).replace("\n", " ").replace("  ", " ").strip()
             for c in hdr_cells
         ]
-        TOOLS.set_small_glyph_heights(old_small)
         return TableHeader(tuple(hdr_bbox), hdr_cells, hdr_names, True)
 
 
@@ -1820,8 +1843,6 @@ page information themselves.
 def make_chars(page, clip=None):
     """Extract text as "rawdict" to fill CHARS."""
     global CHARS
-    old_small = TOOLS.set_small_glyph_heights()
-    TOOLS.set_small_glyph_heights(True)
     page_number = page.number + 1
     page_height = page.rect.height
     ctm = page.transformation_matrix
@@ -1872,53 +1893,132 @@ def make_chars(page, clip=None):
                     }
                     CHARS.append(char_dict)
 
-    TOOLS.set_small_glyph_heights(old_small)
-
 
 # -----------------------------------------------------------------------------
-# Extract all page vector graphics to fill the EDGES list
+# Extract all page vector graphics to fill the EDGES list.
+# We are ignoring Bézier curves completely and are converting everything
+# else to lines.
 # -----------------------------------------------------------------------------
-def make_edges(page, clip=None, tset=None):
+def make_edges(page, clip=None, tset=None, add_lines=None):
     global EDGES
-    paths = page.get_drawings()
+    snap_x = tset.snap_x_tolerance
+    snap_y = tset.snap_y_tolerance
+    lines_strict = (
+        tset.vertical_strategy == "lines_strict"
+        or tset.horizontal_strategy == "lines_strict"
+    )
     page_height = page.rect.height
     doctop_basis = page.number * page_height
     page_number = page.number + 1
-    x_tolerance = tset.snap_x_tolerance
-    y_tolerance = tset.snap_y_tolerance
-    if clip != None:
+    prect = page.rect
+    if page.rotation in (90, 270):
+        w, h = prect.br
+        prect = Rect(0, 0, h, w)
+    if clip is not None:
         clip = Rect(clip)
     else:
-        clip = page.rect
+        clip = prect
+
+    def are_neighbors(r1, r2):
+        """Detect whether r1, r2 are neighbors.
+
+        Defined as:
+        The minimum distance between points of r1 and points of r2 is not
+        larger than some delta.
+
+        This check supports empty rect-likes and thus also lines.
+        """
+        if (
+            r2.x0 - snap_x <= r1.x0 <= r2.x1 + snap_x
+            or r2.x0 - snap_x <= r1.x1 <= r2.x1 + snap_x
+        ) and (
+            r2.y0 - snap_y <= r1.y0 <= r2.y1 + snap_y
+            or r2.y0 - snap_y <= r1.y1 <= r2.y1 + snap_y
+        ):
+            return True
+        return False
+
+    def clean_graphics():
+        """Detect and join rectangles of connected vector graphics."""
+        # exclude irrelevant graphics
+        paths = []
+        for p in page.get_drawings():
+            if (  # ignore fill-only graphics if they are no lines
+                p["type"] == "f"
+                and lines_strict
+                and p["rect"].width > snap_x
+                and p["rect"].height > snap_y
+            ):
+                continue
+            paths.append(p)
+
+        prects = sorted([p["rect"] for p in paths], key=lambda r: (r.y1, r.x0))
+        new_rects = []  # the final list of joined rectangles
+        # -------------------------------------------------------------------------
+        # Strategy: Join rectangles that "almost touch" each other,
+        # Extend first rectangle with any remaining in the list that touches it.
+        # Then move it to final list and continue with the rest.
+        # -------------------------------------------------------------------------
+        while prects:  # the algorithm will empty this list
+            r = prects[0]  # first rectangle
+            repeat = True
+            while repeat:  # this loop extends first rect in list
+                repeat = False  # will be set to true if any other rect touches
+                for i in range(len(prects) - 1, -1, -1):  # run backwards
+                    if i == 0:  # don't touch first rectangle
+                        continue
+                    if are_neighbors(r, prects[i]):  # touches rect 0!
+                        r |= prects[i]  # extend first rect
+                        prects[0] = +r  # update it in list
+                        del prects[i]  # delete this rect
+                        repeat = True  # check remaining
+
+            # move first item over to result list
+            new_rects.append(prects.pop(0))
+            prects = sorted(list(set(prects)), key=lambda r: (r.y1, r.x0))
+
+        new_rects = sorted(list(set(new_rects)), key=lambda r: (r.y1, r.x0))
+        return [r for r in new_rects if r.width > 5 and r.height > 5], paths
+
+    bboxes, paths = clean_graphics()
+
+    def is_parallel(p1, p2):
+        """Check if line is roughly axis-parallel."""
+        if abs(p1.x - p2.x) <= snap_x or abs(p1.y - p2.y) <= snap_y:
+            return True
+        return False
 
     def make_line(p, p1, p2, clip):
+        """Given 2 points, make a line dictionary for table detection."""
+        if not is_parallel(p1, p2):  # only accepting axis-parallel lines
+            return {}
+        # compute the extremal values
         x0 = min(p1.x, p2.x)
         x1 = max(p1.x, p2.x)
         y0 = min(p1.y, p2.y)
         y1 = max(p1.y, p2.y)
 
-        if x0 > clip.x1:
+        # check for outside clip
+        if x0 > clip.x1 or x1 < clip.x0 or y0 > clip.y1 or y1 < clip.y0:
             return {}
-        if x0 < clip.x0:
-            x0 = clip.x0
-        if x1 < clip.x0:
-            return {}
-        if x1 > clip.x1:
-            x1 = clip.x1
-        if y0 > clip.y1:
-            return {}
-        if y0 < clip.y0:
-            y0 = clip.y0
-        if y1 < clip.y0:
-            return {}
-        if y1 > clip.y1:
-            y1 = clip.y1
 
-        width = x1 - x0
-        height = y1 - y0
+        if x0 < clip.x0:
+            x0 = clip.x0  # adjust to clip boundary
+
+        if x1 > clip.x1:
+            x1 = clip.x1  # adjust to clip boundary
+
+        if y0 < clip.y0:
+            y0 = clip.y0  # adjust to clip boundary
+
+        if y1 > clip.y1:
+            y1 = clip.y1  # adjust to clip boundary
+
+        width = x1 - x0  # from adjusted values
+        height = y1 - y0  # from adjusted values
         if width == height == 0:
-            return {}
-        return {
+            return {}  # nothing left to deal with
+        line_dict = {
             "x0": x0,
             "y0": page_height - y0,
             "x1": x1,
@@ -1940,110 +2040,170 @@ def make_edges(page, clip=None, tset=None):
             "bottom": y1,
             "doctop": y0 + doctop_basis,
         }
+        return line_dict
 
     for p in paths:
-        for i in p["items"]:
+        items = p["items"]  # items in this path
+
+        # if 'closePath', add a line from last to first point
+        if p["closePath"] and items[0][0] == "l" and items[-1][0] == "l":
+            items.append(("l", items[-1][2], items[0][1]))
+
+        for i in items:
             if i[0] not in ("l", "re", "qu"):
-                continue
-            if i[0] == "l":
+                continue  # ignore anything else
+
+            if i[0] == "l":  # a line
                 p1, p2 = i[1:]
-                if p1.x != p2.x and p1.y != p2.y:
-                    # ignore lines not parallel to either axis
-                    continue
                 line_dict = make_line(p, p1, p2, clip)
-                if line_dict != {}:
+                if line_dict:
                     EDGES.append(line_to_edge(line_dict))
-            elif i[0] == "re":
-                rect = i[1]
+
+            elif i[0] == "re":  # a rectangle: decompose into 4 lines
+                rect = i[1].normalize()  # rectangle itself
                 # ignore minute rectangles
-                if rect.height <= y_tolerance and rect.width <= x_tolerance:
+                if rect.height <= snap_y and rect.width <= snap_x:
                     continue
-                if rect.width <= x_tolerance:
-                    x = abs(rect.x1 + rect.x0) / 2
+
+                if rect.width <= snap_x:  # simulates a vertical line
+                    x = abs(rect.x1 + rect.x0) / 2  # take middle value for x
                     p1 = Point(x, rect.y0)
                     p2 = Point(x, rect.y1)
                     line_dict = make_line(p, p1, p2, clip)
-                    if line_dict != {}:
-                        EDGES.append(line_to_edge(line_dict))
-                    continue
-                elif rect.height <= y_tolerance:
-                    y = abs(rect.y1 + rect.y0) / 2
-                    p1 = Point(rect.x0, y)
-                    p2 = Point(rect.x1, y)
-                    line_dict = make_line(p, p1, p2, clip)
-                    if line_dict != {}:
+                    if line_dict:
                         EDGES.append(line_to_edge(line_dict))
                     continue
 
-                if clip != None:
-                    rect &= clip
-                rdict = {
-                    "x0": rect.x0,
-                    "y0": page_height - rect.y1,
-                    "x1": rect.x1,
-                    "y1": page_height - rect.y0,
-                    "width": rect.width,
-                    "height": rect.height,
-                    "pts": [
-                        tuple(rect.bl),
-                        tuple(rect.br),
-                        tuple(rect.tr),
-                        tuple(rect.tl),
-                        tuple(rect.bl),
-                        tuple(rect.bl),
-                    ],
-                    "linewidth": p["width"],
-                    "stroke": True if "s" in p["type"] else False,
-                    "fill": True if "f" in p["type"] else False,
-                    "evenodd": False,
-                    "stroking_color": p["color"],
-                    "non_stroking_color": p["fill"],
-                    "object_type": "curve",
-                    "page_number": page_number,
-                    "stroking_pattern": None,
-                    "non_stroking_pattern": None,
-                    "top": rect.y0,
-                    "bottom": rect.y1,
-                    "doctop": doctop_basis + rect.y0,
-                }
-                if not rect.is_empty:
-                    EDGES.extend(curve_to_edges(rdict))
-            else:
-                quad = i[1]
-                rect = quad.rect
-                if clip != None and rect not in clip:
+                if rect.height <= snap_y:  # simulates a horizontal line
+                    y = abs(rect.y1 + rect.y0) / 2  # take middle value for y
+                    p1 = Point(rect.x0, y)
+                    p2 = Point(rect.x1, y)
+                    line_dict = make_line(p, p1, p2, clip)
+                    if line_dict:
+                        EDGES.append(line_to_edge(line_dict))
                     continue
+
+                line_dict = make_line(p, rect.tl, rect.bl, clip)
+                if line_dict:
+                    EDGES.append(line_to_edge(line_dict))
+
+                line_dict = make_line(p, rect.bl, rect.br, clip)
+                if line_dict:
+                    EDGES.append(line_to_edge(line_dict))
+
+                line_dict = make_line(p, rect.br, rect.tr, clip)
+                if line_dict:
+                    EDGES.append(line_to_edge(line_dict))
+
+                line_dict = make_line(p, rect.tr, rect.tl, clip)
+                if line_dict:
+                    EDGES.append(line_to_edge(line_dict))
+
+            else:  # must be a quad
+                # we convert it into (up to) 4 lines
                 ul, ur, ll, lr = i[1]
-                qdict = {
-                    "bottom": rect.y1,
-                    "doctop": rect.y0 + doctop_basis,
-                    "evenodd": p["even_odd"],
-                    "fill": True if "f" in p["type"] else False,
-                    "height": rect.height,
-                    "linewidth": p["width"],
-                    "non_stroking_color": p["fill"],
-                    "non_stroking_pattern": None,
-                    "object_type": "curve",
-                    "page_number": page_number,
-                    "pts": [
-                        tuple(ul),
-                        tuple(ll),
-                        tuple(lr),
-                        tuple(ur),
-                        tuple(ul),
-                        tuple(ul),
-                    ],
-                    "stroke": True if "s" in p["type"] else False,
-                    "stroking_color": p["color"],
-                    "stroking_pattern": None,
-                    "top": rect.y0,
-                    "width": rect.width,
-                    "x0": rect.x0,
-                    "x1": rect.x1,
-                    "y0": page_height - rect.y1,
-                    "y1": page_height - rect.y0,
-                }
-                EDGES.extend(curve_to_edges(qdict))
+
+                line_dict = make_line(p, ul, ll, clip)
+                if line_dict:
+                    EDGES.append(line_to_edge(line_dict))
+
+                line_dict = make_line(p, ll, lr, clip)
+                if line_dict:
+                    EDGES.append(line_to_edge(line_dict))
+
+                line_dict = make_line(p, lr, ur, clip)
+                if line_dict:
+                    EDGES.append(line_to_edge(line_dict))
+
+                line_dict = make_line(p, ur, ul, clip)
+                if line_dict:
+                    EDGES.append(line_to_edge(line_dict))
+
+    path = {"color": (0, 0, 0), "fill": None, "width": 1}
+    for bbox in bboxes:  # add the border lines for all enveloping bboxes
+        line_dict = make_line(path, bbox.tl, bbox.tr, clip)
+        if line_dict:
+            EDGES.append(line_to_edge(line_dict))
+
+        line_dict = make_line(path, bbox.bl, bbox.br, clip)
+        if line_dict:
+            EDGES.append(line_to_edge(line_dict))
+
+        line_dict = make_line(path, bbox.tl, bbox.bl, clip)
+        if line_dict:
+            EDGES.append(line_to_edge(line_dict))
+
+        line_dict = make_line(path, bbox.tr, bbox.br, clip)
+        if line_dict:
+            EDGES.append(line_to_edge(line_dict))
+
+    if add_lines is not None:  # add user-specified lines
+        assert isinstance(add_lines, (tuple, list))
+    else:
+        add_lines = []
+    for p1, p2 in add_lines:
+        p1 = Point(p1)
+        p2 = Point(p2)
+        line_dict = make_line(path, p1, p2, clip)
+        if line_dict:
+            EDGES.append(line_to_edge(line_dict))
+
+
+def page_rotation_set0(page):
+    """Nullify page rotation.
+
+    To correctly detect tables, page rotation must be zero.
+    This function performs the necessary adjustments and returns information
+    for reverting this changes.
+    """
+    mediabox = page.mediabox
+    rot = page.rotation  # contains normalized rotation value
+    # need to derotate the page's content
+    mb = page.mediabox  # current mediabox
+
+    if rot == 90:
+        # before derotation, shift content horizontally
+        mat0 = Matrix(1, 0, 0, 1, mb.y1 - mb.x1 - mb.x0 - mb.y0, 0)
+    elif rot == 270:
+        # before derotation, shift content vertically
+        mat0 = Matrix(1, 0, 0, 1, 0, mb.x1 - mb.y1 - mb.y0 - mb.x0)
+    else:
+        mat0 = Matrix(1, 0, 0, 1, -2 * mb.x0, -2 * mb.y0)
+
+    # prefix with derotation matrix
+    mat = mat0 * page.derotation_matrix
+    cmd = b"%g %g %g %g %g %g cm " % tuple(mat)
+    xref = TOOLS._insert_contents(page, cmd, 0)
+
+    # swap x- and y-coordinates
+    if rot in (90, 270):
+        x0, y0, x1, y1 = mb
+        mb.x0 = y0
+        mb.y0 = x0
+        mb.x1 = y1
+        mb.y1 = x1
+        page.set_mediabox(mb)
+
+    page.set_rotation(0)
+
+    # refresh the page to apply these changes
+    doc = page.parent
+    pno = page.number
+    page = doc[pno]
+    return page, xref, rot, mediabox
+
+
+def page_rotation_reset(page, xref, rot, mediabox):
+    """Reset page rotation to original values.
+
+    To be used before we return tables."""
+    doc = page.parent  # document of the page
+    doc.update_stream(xref, b" ")  # remove de-rotation matrix
+    page.set_mediabox(mediabox)  # set mediabox to old value
+    page.set_rotation(rot)  # set rotation to old value
+    pno = page.number
+    page = doc[pno]  # update page info
+    return page
 
 
 def find_tables(
@@ -2068,22 +2228,35 @@ def find_tables(
     text_tolerance=3,
     text_x_tolerance=3,
     text_y_tolerance=3,
+    strategy=None,  # offer abbreviation
+    add_lines=None,  # optional user-specified lines
 ):
     global CHARS, EDGES
     CHARS = []
     EDGES = []
-    if snap_x_tolerance == None:
+    old_small = bool(TOOLS.set_small_glyph_heights())  # save old value
+    TOOLS.set_small_glyph_heights(True)  # we need minimum bboxes
+    if page.rotation != 0:
+        page, old_xref, old_rot, old_mediabox = page_rotation_set0(page)
+    else:
+        old_xref, old_rot, old_mediabox = None, None, None
+
+    if snap_x_tolerance is None:
         snap_x_tolerance = UNSET
-    if snap_y_tolerance == None:
+    if snap_y_tolerance is None:
         snap_y_tolerance = UNSET
-    if join_x_tolerance == None:
+    if join_x_tolerance is None:
         join_x_tolerance = UNSET
-    if join_y_tolerance == None:
+    if join_y_tolerance is None:
         join_y_tolerance = UNSET
-    if intersection_x_tolerance == None:
+    if intersection_x_tolerance is None:
         intersection_x_tolerance = UNSET
-    if intersection_y_tolerance == None:
+    if intersection_y_tolerance is None:
         intersection_y_tolerance = UNSET
+    if strategy is not None:
+        vertical_strategy = strategy
+        horizontal_strategy = strategy
+
     settings = {
         "vertical_strategy": vertical_strategy,
         "horizontal_strategy": horizontal_strategy,
@@ -2107,7 +2280,14 @@ def find_tables(
     }
     tset = TableSettings.resolve(settings=settings)
     page.table_settings = tset
+
     make_chars(page, clip=clip)  # create character list of page
-    make_edges(page, clip=clip, tset=tset)  # create lines and curves
+    make_edges(
+        page, clip=clip, tset=tset, add_lines=add_lines
+    )  # create lines and curves
     tables = TableFinder(page, settings=tset)
+
+    TOOLS.set_small_glyph_heights(old_small)
+    if old_xref is not None:
+        page = page_rotation_reset(page, old_xref, old_rot, old_mediabox)
     return tables
